/* This file is used to validate the scan orders in Sec. 6.5.1 and 6.5.2 of the WD */

#include <iostream>

void printf_array(int *arr, int n)
{
  for(int i = 0; i < n; i++)
    printf("%4d", arr[i]);
  printf("\n");
}
void printf_matrix(int arr[200][200], int n1, int n2)
{
  for(int i = 0; i < n1; i++)
    printf_array(arr[i], n2);
  printf("\n");
}
void printf_matrix1(int *arr, int n1, int n2)
{
	for(int j = 0; j < n2; j++)
	{
		for(int i = 0; i < n1; i++)
			printf("%4d",arr[j*n1+i]);
		printf("\n");
	}
}

int main()
{
  const int len = 10;
  const int lenB = 200;
  int picHeightInCtbs, picWidthInCtbs;
  int num_tile_columns_minus1, num_tile_rows_minus1;
  int tbX, tbY, tileX, tileY;
  int ctbAddrRS, i, j, x, y, tIdx;
  int rowBd[len], colBd[len], rowHeight[len], colWidth[len];
  int ctbAddrRStoTS[lenB], ctbAddrTStoRS[lenB], tileId[lenB];
  int ctbAddrTStoRS1[lenB], tileId1[lenB];
  
  int picHeightInMinCbs, picWidthInMinCbs, log2MinCbSize, log2CtbSize, log2_diff_max_min_coding_block_size;
  int m, p;
  int minCbAddrZS[lenB][lenB];

  // -----------------------------------------------------------------
  // Initializing the values
  // -----------------------------------------------------------------
  num_tile_columns_minus1 = 2;
  num_tile_rows_minus1    = 1;  

  colWidth[0] = 2; colWidth[1] = 3; colWidth[2] = 2;
  rowHeight[0] = 3; rowHeight[1] = 3;

  log2MinCbSize     = 4;
  log2CtbSize       = 5;

  // Print input conditions:
  printf("num_tile_columns_minus1   : %d\n", num_tile_columns_minus1);
  printf("num_tile_rows_minus1      : %d\n", num_tile_rows_minus1);
  printf("colWidth                  : ");
  for(i = 0; i <= num_tile_columns_minus1; i++)
    printf("%d ", colWidth[i]);
  printf("\n");
  printf("rowHeight                 : ");
  for(i = 0; i <= num_tile_rows_minus1; i++)
    printf("%d ", rowHeight[i]);
  printf("\n");
  printf("log2MinCbSize             : %d\n", log2MinCbSize);
  printf("log2CtbSize               : %d\n", log2CtbSize);
  // -----------------------------------------------------------------

  for(picWidthInCtbs = 0, i = 0; i <= num_tile_columns_minus1; i++ )
    picWidthInCtbs += colWidth[i];
  for(picHeightInCtbs = 0, i = 0; i <= num_tile_rows_minus1; i++ )
    picHeightInCtbs += rowHeight[i];
  
  for( colBd[ 0 ] = 0, i = 0; i <= num_tile_columns_minus1; i++ )
		colBd[ i + 1 ] = colBd[ i ] + colWidth[ i ];
  
  for( rowBd[ 0 ] = 0, j = 0; j <= num_tile_rows_minus1; j++ )
	  rowBd[ j + 1 ] = rowBd[ j ] + rowHeight[ j ];  
  
  log2_diff_max_min_coding_block_size = log2CtbSize - log2MinCbSize;
  picHeightInMinCbs = picHeightInCtbs * (1 << log2_diff_max_min_coding_block_size);
  picWidthInMinCbs  = picWidthInCtbs * (1 << log2_diff_max_min_coding_block_size);

  /* -----------------------------------
        Section 6.5.1
   ----------------------------------- */

	for( ctbAddrRS = 0; ctbAddrRS < picHeightInCtbs * picWidthInCtbs; ctbAddrRS++ ) 
  {
		tbX = ctbAddrRS % picWidthInCtbs;
		tbY = ctbAddrRS / picWidthInCtbs;
		for( i = 0; i <= num_tile_columns_minus1; i++ )
			if( tbX >= colBd[ i ] )
				tileX = i;
		ctbAddrRStoTS[ ctbAddrRS ] = tbY * colWidth[ tileX ];
		for( i = 0; i < tileX; i++ )
			ctbAddrRStoTS[ ctbAddrRS ] += picHeightInCtbs * colWidth[ i ];
		ctbAddrRStoTS[ ctbAddrRS ] += tbX - colBd[ tileX ];


    // ctbAddrTStoRS1[ ctbAddrRStoTS[ ctbAddrRS ] ] = ctbAddrRS;
    // tileId1[ ctbAddrRStoTS[ ctbAddrRS ] ] = tileX + tileY * (num_tile_columns_minus1 + 1);
  }

  printf("ctrAddrRStoTS:\n");
  printf_matrix1(ctbAddrRStoTS, picWidthInCtbs, picHeightInCtbs);


    for( i = 0, tIdx = 0; i <= num_tile_columns_minus1; i++)
        for( j = 0; j <= num_tile_rows_minus1; j++, tIdx++ )
			for( y = rowBd[ j ]; y < rowBd[ j + 1 ]; y++ )
				for( x = colBd[ i ]; x < colBd[ i + 1 ]; x++ )
						tileId[ ctbAddrRStoTS[ y*picWidthInCtbs + x ] ] = tIdx;

	printf("tileId\n");

	for(int j = 0; j < picHeightInCtbs; j++)
	{
		for(int i = 0; i < picWidthInCtbs; i++)
			printf("%4d",tileId[ctbAddrRStoTS[j*picWidthInCtbs+i]]);
		printf("\n");
	}

	for( ctbAddrRS = 0; ctbAddrRS < picHeightInCtbs * picWidthInCtbs; ctbAddrRS++ )
		ctbAddrTStoRS[ ctbAddrRStoTS[ ctbAddrRS ] ] = ctbAddrRS;


  printf("ctrAddrTStoRS:\n");
  printf_array(ctbAddrTStoRS, picHeightInCtbs * picWidthInCtbs);

    /* -----------------------------------
        Section 6.5.2
   ----------------------------------- */

	for( y = 0; y < picHeightInMinCbs; y++ )
  {
		for( x = 0; x < picWidthInMinCbs; x++) {
			tbX = ( x << log2MinCbSize ) >> log2CtbSize;
			tbY = ( y << log2MinCbSize ) >> log2CtbSize;
			ctbAddrRS = picWidthInCtbs * tbY + tbX;
			minCbAddrZS[ x ][ y ] = ctbAddrRStoTS[ ctbAddrRS ] << (log2_diff_max_min_coding_block_size*2);
			for( i = 0, p = 0; i < log2_diff_max_min_coding_block_size; i++ ) {
				m = 1 << i;
				p += ( m & x ? m*m : 0 ) + ( m & y ? 2 * m * m : 0 );
			}
   		minCbAddrZS[ x ][ y ] += p;
		}
  }
  printf("minCbAddrZS\n");
  // printf_matrix(minCbAddrZS, picWidthInMinCbs, picHeightInMinCbs);

	for(int i = 0; i < picHeightInMinCbs; i++)
	{
  		for(int j = 0; j < picWidthInMinCbs; j++)
			printf("%4d",minCbAddrZS[j][i]);
		printf("\n");
	}
		
  return 0;
}


